package xyz.lychee.lagfixer.libs.adventure.text.event;

import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Objects;
import java.util.UUID;
import java.util.function.UnaryOperator;
import java.util.stream.Stream;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.key.Keyed;
import xyz.lychee.lagfixer.libs.adventure.nbt.api.BinaryTagHolder;
import xyz.lychee.lagfixer.libs.adventure.text.Component;
import xyz.lychee.lagfixer.libs.adventure.text.ComponentLike;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.adventure.text.format.StyleBuilderApplicable;
import xyz.lychee.lagfixer.libs.adventure.text.renderer.ComponentRenderer;
import xyz.lychee.lagfixer.libs.adventure.util.Index;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;
import xyz.lychee.lagfixer.libs.annotations.Range;
import xyz.lychee.lagfixer.libs.annotations.ApiStatus.ScheduledForRemoval;
import xyz.lychee.lagfixer.libs.examination.Examinable;
import xyz.lychee.lagfixer.libs.examination.ExaminableProperty;

public final class HoverEvent<V> implements Examinable, HoverEventSource<V>, StyleBuilderApplicable {
   private final Action<V> action;
   private final V value;

   public static @NotNull HoverEvent<Component> showText(final @NotNull ComponentLike text) {
      return showText(text.asComponent());
   }

   public static @NotNull HoverEvent<Component> showText(final @NotNull Component text) {
      return new HoverEvent(HoverEvent.Action.SHOW_TEXT, text);
   }

   public static @NotNull HoverEvent<ShowItem> showItem(final @NotNull Key item, final int count) {
      return showItem((Keyed)item, count, (Map)Collections.emptyMap());
   }

   public static @NotNull HoverEvent<ShowItem> showItem(final @NotNull Keyed item, final int count) {
      return showItem(item, count, Collections.emptyMap());
   }

   /** @deprecated */
   @Deprecated
   public static @NotNull HoverEvent<ShowItem> showItem(final @NotNull Key item, final int count, final @Nullable BinaryTagHolder nbt) {
      return showItem(HoverEvent.ShowItem.showItem(item, count, nbt));
   }

   /** @deprecated */
   @Deprecated
   public static @NotNull HoverEvent<ShowItem> showItem(final @NotNull Keyed item, final int count, final @Nullable BinaryTagHolder nbt) {
      return showItem(HoverEvent.ShowItem.showItem(item, count, nbt));
   }

   public static @NotNull HoverEvent<ShowItem> showItem(final @NotNull Keyed item, final int count, final @NotNull Map<Key, ? extends DataComponentValue> dataComponents) {
      return showItem(HoverEvent.ShowItem.showItem(item, count, dataComponents));
   }

   public static @NotNull HoverEvent<ShowItem> showItem(@NotNull final @NotNull ShowItem item) {
      return new HoverEvent(HoverEvent.Action.SHOW_ITEM, item);
   }

   public static @NotNull HoverEvent<ShowEntity> showEntity(final @NotNull Key type, final @NotNull UUID id) {
      return showEntity((Key)type, id, (Component)null);
   }

   public static @NotNull HoverEvent<ShowEntity> showEntity(final @NotNull Keyed type, final @NotNull UUID id) {
      return showEntity((Keyed)type, id, (Component)null);
   }

   public static @NotNull HoverEvent<ShowEntity> showEntity(final @NotNull Key type, final @NotNull UUID id, final @Nullable Component name) {
      return showEntity(HoverEvent.ShowEntity.of(type, id, name));
   }

   public static @NotNull HoverEvent<ShowEntity> showEntity(final @NotNull Keyed type, final @NotNull UUID id, final @Nullable Component name) {
      return showEntity(HoverEvent.ShowEntity.of(type, id, name));
   }

   public static @NotNull HoverEvent<ShowEntity> showEntity(@NotNull final @NotNull ShowEntity entity) {
      return new HoverEvent(HoverEvent.Action.SHOW_ENTITY, entity);
   }

   /** @deprecated */
   @Deprecated
   public static @NotNull HoverEvent<String> showAchievement(final @NotNull String value) {
      return new HoverEvent(HoverEvent.Action.SHOW_ACHIEVEMENT, value);
   }

   public static <V> @NotNull HoverEvent<V> hoverEvent(@NotNull final @NotNull Action<V> action, final @NotNull V value) {
      return new HoverEvent(action, value);
   }

   private HoverEvent(@NotNull final @NotNull Action<V> action, final @NotNull V value) {
      this.action = (Action)Objects.requireNonNull(action, "action");
      this.value = Objects.requireNonNull(value, "value");
   }

   @NotNull
   public @NotNull Action<V> action() {
      return this.action;
   }

   public @NotNull V value() {
      return this.value;
   }

   public @NotNull HoverEvent<V> value(final @NotNull V value) {
      return new HoverEvent(this.action, value);
   }

   public <C> @NotNull HoverEvent<V> withRenderedValue(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context) {
      V oldValue = this.value;
      V newValue = this.action.renderer.render(renderer, context, oldValue);
      return newValue != oldValue ? new HoverEvent(this.action, newValue) : this;
   }

   public @NotNull HoverEvent<V> asHoverEvent() {
      return this;
   }

   public @NotNull HoverEvent<V> asHoverEvent(final @NotNull UnaryOperator<V> op) {
      return op == UnaryOperator.identity() ? this : new HoverEvent(this.action, op.apply(this.value));
   }

   public void styleApply(final Style.@NotNull Builder style) {
      style.hoverEvent(this);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (other != null && this.getClass() == other.getClass()) {
         HoverEvent<?> that = (HoverEvent)other;
         return this.action == that.action && this.value.equals(that.value);
      } else {
         return false;
      }
   }

   public int hashCode() {
      int result = this.action.hashCode();
      result = 31 * result + this.value.hashCode();
      return result;
   }

   public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
      return Stream.of(ExaminableProperty.of("action", this.action), ExaminableProperty.of("value", this.value));
   }

   public String toString() {
      return Internals.toString(this);
   }

   public static final class Action<V> {
      public static final Action<Component> SHOW_TEXT = new Action("show_text", Component.class, true, new Renderer<Component>() {
         public <C> @NotNull Component render(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context, final @NotNull Component value) {
            return renderer.render(value, context);
         }
      });
      public static final Action<ShowItem> SHOW_ITEM = new Action("show_item", ShowItem.class, true, new Renderer<ShowItem>() {
         @NotNull
         public <C> @NotNull ShowItem render(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context, @NotNull final @NotNull ShowItem value) {
            return value;
         }
      });
      public static final Action<ShowEntity> SHOW_ENTITY = new Action("show_entity", ShowEntity.class, true, new Renderer<ShowEntity>() {
         @NotNull
         public <C> @NotNull ShowEntity render(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context, @NotNull final @NotNull ShowEntity value) {
            return value.name == null ? value : value.name(renderer.render(value.name, context));
         }
      });
      /** @deprecated */
      @Deprecated
      public static final Action<String> SHOW_ACHIEVEMENT = new Action("show_achievement", String.class, true, new Renderer<String>() {
         public <C> @NotNull String render(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context, final @NotNull String value) {
            return value;
         }
      });
      public static final Index<String, Action<?>> NAMES;
      private final String name;
      private final Class<V> type;
      private final boolean readable;
      private final Renderer<V> renderer;

      Action(final String name, final Class<V> type, final boolean readable, final Renderer<V> renderer) {
         this.name = name;
         this.type = type;
         this.readable = readable;
         this.renderer = renderer;
      }

      public @NotNull Class<V> type() {
         return this.type;
      }

      public boolean readable() {
         return this.readable;
      }

      public @NotNull String toString() {
         return this.name;
      }

      static {
         NAMES = Index.create((constant) -> {
            return constant.name;
         }, new Action[]{SHOW_TEXT, SHOW_ITEM, SHOW_ENTITY, SHOW_ACHIEVEMENT});
      }

      @FunctionalInterface
      interface Renderer<V> {
         <C> @NotNull V render(final @NotNull ComponentRenderer<C> renderer, final @NotNull C context, final @NotNull V value);
      }
   }

   public static final class ShowEntity implements Examinable {
      private final Key type;
      private final UUID id;
      private final Component name;

      @NotNull
      public static @NotNull ShowEntity showEntity(final @NotNull Key type, final @NotNull UUID id) {
         return showEntity((Key)type, id, (Component)null);
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowEntity of(final @NotNull Key type, final @NotNull UUID id) {
         return of((Key)type, id, (Component)null);
      }

      @NotNull
      public static @NotNull ShowEntity showEntity(final @NotNull Keyed type, final @NotNull UUID id) {
         return showEntity((Keyed)type, id, (Component)null);
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowEntity of(final @NotNull Keyed type, final @NotNull UUID id) {
         return of((Keyed)type, id, (Component)null);
      }

      @NotNull
      public static @NotNull ShowEntity showEntity(final @NotNull Key type, final @NotNull UUID id, final @Nullable Component name) {
         return new ShowEntity((Key)Objects.requireNonNull(type, "type"), (UUID)Objects.requireNonNull(id, "id"), name);
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowEntity of(final @NotNull Key type, final @NotNull UUID id, final @Nullable Component name) {
         return new ShowEntity((Key)Objects.requireNonNull(type, "type"), (UUID)Objects.requireNonNull(id, "id"), name);
      }

      @NotNull
      public static @NotNull ShowEntity showEntity(final @NotNull Keyed type, final @NotNull UUID id, final @Nullable Component name) {
         return new ShowEntity(((Keyed)Objects.requireNonNull(type, "type")).key(), (UUID)Objects.requireNonNull(id, "id"), name);
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowEntity of(final @NotNull Keyed type, final @NotNull UUID id, final @Nullable Component name) {
         return new ShowEntity(((Keyed)Objects.requireNonNull(type, "type")).key(), (UUID)Objects.requireNonNull(id, "id"), name);
      }

      private ShowEntity(final @NotNull Key type, final @NotNull UUID id, final @Nullable Component name) {
         this.type = type;
         this.id = id;
         this.name = name;
      }

      public @NotNull Key type() {
         return this.type;
      }

      @NotNull
      public @NotNull ShowEntity type(final @NotNull Key type) {
         return ((Key)Objects.requireNonNull(type, "type")).equals(this.type) ? this : new ShowEntity(type, this.id, this.name);
      }

      @NotNull
      public @NotNull ShowEntity type(final @NotNull Keyed type) {
         return this.type(((Keyed)Objects.requireNonNull(type, "type")).key());
      }

      public @NotNull UUID id() {
         return this.id;
      }

      @NotNull
      public @NotNull ShowEntity id(final @NotNull UUID id) {
         return ((UUID)Objects.requireNonNull(id)).equals(this.id) ? this : new ShowEntity(this.type, id, this.name);
      }

      public @Nullable Component name() {
         return this.name;
      }

      @NotNull
      public @NotNull ShowEntity name(final @Nullable Component name) {
         return Objects.equals(name, this.name) ? this : new ShowEntity(this.type, this.id, name);
      }

      public boolean equals(final @Nullable Object other) {
         if (this == other) {
            return true;
         } else if (other != null && this.getClass() == other.getClass()) {
            ShowEntity that = (ShowEntity)other;
            return this.type.equals(that.type) && this.id.equals(that.id) && Objects.equals(this.name, that.name);
         } else {
            return false;
         }
      }

      public int hashCode() {
         int result = this.type.hashCode();
         result = 31 * result + this.id.hashCode();
         result = 31 * result + Objects.hashCode(this.name);
         return result;
      }

      public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
         return Stream.of(ExaminableProperty.of("type", this.type), ExaminableProperty.of("id", this.id), ExaminableProperty.of("name", this.name));
      }

      public String toString() {
         return Internals.toString(this);
      }
   }

   public static final class ShowItem implements Examinable {
      private final Key item;
      private final int count;
      private final @Nullable BinaryTagHolder nbt;
      private final Map<Key, DataComponentValue> dataComponents;

      @NotNull
      public static @NotNull ShowItem showItem(final @NotNull Key item, final @Range(
   from = 0L,
   to = 2147483647L
) int count) {
         return showItem((Keyed)item, count, (Map)Collections.emptyMap());
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowItem of(final @NotNull Key item, final @Range(
   from = 0L,
   to = 2147483647L
) int count) {
         return showItem((Keyed)item, count, (Map)Collections.emptyMap());
      }

      @NotNull
      public static @NotNull ShowItem showItem(final @NotNull Keyed item, final @Range(
   from = 0L,
   to = 2147483647L
) int count) {
         return showItem(item, count, Collections.emptyMap());
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowItem of(final @NotNull Keyed item, final @Range(
   from = 0L,
   to = 2147483647L
) int count) {
         return of((Keyed)item, count, (BinaryTagHolder)null);
      }

      /** @deprecated */
      @Deprecated
      @NotNull
      public static @NotNull ShowItem showItem(final @NotNull Key item, final @Range(
   from = 0L,
   to = 2147483647L
) int count, final @Nullable BinaryTagHolder nbt) {
         return new ShowItem((Key)Objects.requireNonNull(item, "item"), count, nbt, Collections.emptyMap());
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowItem of(final @NotNull Key item, final @Range(
   from = 0L,
   to = 2147483647L
) int count, final @Nullable BinaryTagHolder nbt) {
         return new ShowItem((Key)Objects.requireNonNull(item, "item"), count, nbt, Collections.emptyMap());
      }

      /** @deprecated */
      @Deprecated
      @NotNull
      public static @NotNull ShowItem showItem(final @NotNull Keyed item, final @Range(
   from = 0L,
   to = 2147483647L
) int count, final @Nullable BinaryTagHolder nbt) {
         return new ShowItem(((Keyed)Objects.requireNonNull(item, "item")).key(), count, nbt, Collections.emptyMap());
      }

      /** @deprecated */
      @Deprecated
      @ScheduledForRemoval(
         inVersion = "5.0.0"
      )
      @NotNull
      public static @NotNull ShowItem of(final @NotNull Keyed item, final @Range(
   from = 0L,
   to = 2147483647L
) int count, final @Nullable BinaryTagHolder nbt) {
         return new ShowItem(((Keyed)Objects.requireNonNull(item, "item")).key(), count, nbt, Collections.emptyMap());
      }

      @NotNull
      public static @NotNull ShowItem showItem(final @NotNull Keyed item, final int count, final @NotNull Map<Key, ? extends DataComponentValue> dataComponents) {
         return new ShowItem(((Keyed)Objects.requireNonNull(item, "item")).key(), count, (BinaryTagHolder)null, dataComponents);
      }

      private ShowItem(final @NotNull Key item, final int count, final @Nullable BinaryTagHolder nbt, final @NotNull Map<Key, ? extends DataComponentValue> dataComponents) {
         this.item = item;
         this.count = count;
         this.nbt = nbt;
         this.dataComponents = Collections.unmodifiableMap(new HashMap(dataComponents));
      }

      public @NotNull Key item() {
         return this.item;
      }

      @NotNull
      public @NotNull ShowItem item(final @NotNull Key item) {
         return ((Key)Objects.requireNonNull(item, "item")).equals(this.item) ? this : new ShowItem(item, this.count, this.nbt, this.dataComponents);
      }

      public @Range(
   from = 0L,
   to = 2147483647L
) int count() {
         return this.count;
      }

      @NotNull
      public @NotNull ShowItem count(final @Range(
   from = 0L,
   to = 2147483647L
) int count) {
         return count == this.count ? this : new ShowItem(this.item, count, this.nbt, this.dataComponents);
      }

      /** @deprecated */
      @Deprecated
      public @Nullable BinaryTagHolder nbt() {
         return this.nbt;
      }

      /** @deprecated */
      @Deprecated
      @NotNull
      public @NotNull ShowItem nbt(final @Nullable BinaryTagHolder nbt) {
         return Objects.equals(nbt, this.nbt) ? this : new ShowItem(this.item, this.count, nbt, Collections.emptyMap());
      }

      public @NotNull Map<Key, DataComponentValue> dataComponents() {
         return this.dataComponents;
      }

      @NotNull
      public @NotNull ShowItem dataComponents(final @NotNull Map<Key, DataComponentValue> holder) {
         return Objects.equals(this.dataComponents, holder) ? this : new ShowItem(this.item, this.count, (BinaryTagHolder)null, holder.isEmpty() ? Collections.emptyMap() : Collections.unmodifiableMap(new HashMap(holder)));
      }

      public <V extends DataComponentValue> @NotNull Map<Key, V> dataComponentsAs(final @NotNull Class<V> targetType) {
         if (this.dataComponents.isEmpty()) {
            return Collections.emptyMap();
         } else {
            Map<Key, V> results = new HashMap(this.dataComponents.size());
            Iterator var3 = this.dataComponents.entrySet().iterator();

            while(var3.hasNext()) {
               Map.Entry<Key, DataComponentValue> entry = (Map.Entry)var3.next();
               results.put((Key)entry.getKey(), DataComponentValueConverterRegistry.convert(targetType, (Key)entry.getKey(), (DataComponentValue)entry.getValue()));
            }

            return Collections.unmodifiableMap(results);
         }
      }

      public boolean equals(final @Nullable Object other) {
         if (this == other) {
            return true;
         } else if (other != null && this.getClass() == other.getClass()) {
            ShowItem that = (ShowItem)other;
            return this.item.equals(that.item) && this.count == that.count && Objects.equals(this.nbt, that.nbt) && Objects.equals(this.dataComponents, that.dataComponents);
         } else {
            return false;
         }
      }

      public int hashCode() {
         int result = this.item.hashCode();
         result = 31 * result + Integer.hashCode(this.count);
         result = 31 * result + Objects.hashCode(this.nbt);
         result = 31 * result + Objects.hashCode(this.dataComponents);
         return result;
      }

      public @NotNull Stream<? extends ExaminableProperty> examinableProperties() {
         return Stream.of(ExaminableProperty.of("item", this.item), ExaminableProperty.of("count", this.count), ExaminableProperty.of("nbt", this.nbt), ExaminableProperty.of("dataComponents", this.dataComponents));
      }

      public String toString() {
         return Internals.toString(this);
      }
   }
}
